import datetime
import time
from data_mod import DataMod
import sys
import os
import glob
import sets
from dateutil.relativedelta import *
from cdo import *
import copy
from Scientific.N import *
from Scientific.IO.NetCDF import *
import numpy
import shutil
from common import *
#from nco import Nco


class DataEC23(DataMod):
     """ classe of data for the b0cm and b0gq experiments
     """
     def __init__(self, dataname, varname):
         DataMod.__init__(self, dataname, varname)
         self._nmenber=10
         self._freq=datetime.timedelta(0, 0, 0, 0, 0, 6)

     def set_inputdirAuto(self):
         """defining the path to look for the model daily data"""
         dataname=self.get_dataname()
         varname=self.get_varname()
         
         suff="_6hourly"
         inputdir = "/cfunas/exp/ecearth/"+dataname+"/daily_grb/" + varname + suff +"/"

         if (not os.path.isdir(inputdir)):
             print "This directory does not exist:"
             print inputdir
             sys.exit(0)

         self.set_inputdir(inputdir)


     def format_files_nc(self):
         """transform the grb file 
         into ncdf file readable for cdo"""
         
         cdo=Cdo()
         #nco = Nco()

         #get the needed variables from the object
         filelist=self.get_filelist()
         sdates=self.get_sdates()
         dataname=self.get_dataname()
         varname=self.get_varname()
         nmember=self.get_nmember()

         #in b0cm and b0gq the leadtime is 4month
         #hard coded
         nbmon=relativedelta(months=+4, days=-1)
         if varname in ["lsp", "cp"]:
              self.set_freq(datetime.timedelta(0, 0, 0, 0, 0, 24))

         freq=self.get_freq()

         for ifile in range(len(filelist)):
             f=filelist[ifile]
             d1=sdates[ifile]
             
             #calculate the last date of the file
             d2=d1+nbmon
            
             #calculate the number of timestep
             if freq.seconds != 0:
                  nbstep=((d2-d1).days + 1)*86400/freq.seconds + 1
             else:
                  nbstep=((d2-d1).days + 1) + 1
             
             #set the grid 
             f=cdo.setgridtype("regular", input=f,  options="-r -t ecmwf -f nc")

             #list to store the members
             lstmemb=[]
             #loop over the member
             for imemb in range(10):
                  step=[i * 10 +imemb for i in range(nbstep)]
                  #convert step in string
                  step=str(step).strip('[]').replace(" ","")
                  #select one member
                  fmemb=f.split("/")[-1]+str(imemb)
                  #print(fmemb)
                  cdo.seltimestep(step,  
                                  input=f, output=fmemb, options="-r", 
                                  force =  False)
                  lstmemb.append(fmemb)

             #print(lstmemb)
             #cat the ensemble members in a a new dimension
             out="format_"+varname+"_"+str(d1).replace("-","")+".nc"
             lstmemb=str(lstmemb).strip('[]').replace(", "," ").replace("'","")

             dictvar={"tas":"T2M","lsp":"LSP","cp":"CP","psl":"MSL"}
             oldname=dictvar.get(varname)
             if oldname == None:
                  print("the variable "+varname+"is no available in the format_file function")
                  sys.exit(0)
             f=cdo.chvar(oldname,varname, 
                         input=cdo.merge(input=lstmemb,output=out+"tmp",
                                         options="-r", 
                                         force =  False),
                         output=out, options="-r", 
                         force =  False)


             filelist[ifile]=f
         self.set_filelist=list(filelist)
         
     def ready(self, date1, date2):
         """ make the Data object ready to calculate statistic
         """
         cdo=Cdo()

         dataname=self.get_dataname()
         varname=self.get_varname()

         if varname == "tasmin":
              self.set_varname("tas")
              self.set_inputdirAuto()
              self.set_outputdirAuto() #look for the output directory
              self.set_files(date1,date2) #look for the files between date1 and date2 
              self.format_files_nc() #transform grib in nc files cdo compatible
              self.cdo_statistic("daymin","tasmin")
              self.see()
         elif varname == "tasmax":
              self.set_varname("tas")
              self.set_inputdirAuto()
              self.set_outputdirAuto() #look for the output directory
              self.set_files(date1,date2) #look for the files between date1 and date2 
              self.format_files_nc() #transform grib in nc files cdo compatible
              self.cdo_statistic("daymax","tasmax")
         elif varname == "prlr":
              auxlsp=DataEC23(dataname,"lsp")
              auxlsp.set_inputdirAuto()
              auxlsp.set_files(date1,date2) #look for the files between date1 and date2 
              auxlsp.format_files_nc() #transform grib in nc files cdo compatible

              auxcp=DataEC23(dataname,"cp")
              auxcp.set_inputdirAuto()
              auxcp.set_files(date1,date2) #look for the files between date1 and date2 
              auxcp.format_files_nc() #transform grib in nc files cdo compatible

              aux=weighted_sum((auxlsp, auxcp), (1,1), varname)
              aux.set_outputdirAuto() #look for the output directory
              self.copy(aux)
         else:
              self.set_varname(varname)
              self.set_inputdirAuto()
              self.set_outputdirAuto() #look for the output directory
              self.set_files(date1,date2) #look for the files between date1 and date2 
              self.format_files_nc() #transform grib in nc files cdo compatible
              self.see()
