from data import Data
import sys
import os
import time
import datetime
from cdo import *
import copy
from Scientific.N import *
from Scientific.IO.NetCDF import *
from dateutil.relativedelta import *
import numpy
from common import *
import glob
import shutil
from set_output_name import *

class DataObs(Data):

    def __init__(self, dataname, varname):
        Data.__init__(self, dataname, varname)
        self._nmember=1
        self._filetype="nc"

    def duplicate(self):
        """copy all atribute except the lists"""
        exp2=DataObs("","")
        exp2.set_varname(self.get_varname())
        exp2.set_dataname(self.get_dataname())
        exp2.set_freq(self.get_freq())
        exp2.set_inputdir(self.get_inputdir())
        exp2.set_outputdir(self.get_outputdir())
        exp2.set_nmember(self.get_nmember())
        return exp2

    def set_inputdirAuto(self):
        """look for input and output directory corresponding to the dataname"""
        dataname=self.get_dataname() 
        varname=self.get_varname() 
        dataobs = { "ERAint":"/cfu/data/ecmwf/eraint/daily_mean/",
                    "CFS":"/cfunas/exp/CFS/6hourly/"}
        #for 2007 corrupted file in the standard directory

        try:
            inputdir=dataobs.get(dataname) 
        except:
            print "The observations " + dataname  + " is not yet available in the reading_data function"
            sys.exit(0)
        inputdir = inputdir   + varname + "/"
        

        if (not os.path.isdir(inputdir)):
            print "This directory does not exist:"
            print inputdir
            sys.exit(0)
            
        self.set_inputdir(inputdir)

    def set_files(self, date1, date2):
        """create the Filelist and sdates argument with all files
        between date1 and date2 separated by one year
        check if these files exist
        check their type"""
        inputdir=self.get_inputdir()
        filelist=[]
        datelist=[]

        filename=self.get_varname() + "_" + (str(date1).replace("-",""))[0:6]+".nc"
        #we determine the type of file with the first year
        try:
            glob.glob(inputdir+filename)
        except:
            print "the file %s does not exist" % (inputdir+filename)
            print "the directory %s must contain this file:"%inputdir
            print filename
            sys.exit(0)
        filelist.append(inputdir+"/"+filename)
        datelist.append(date1)
        date1 = date1 + relativedelta(years=+1)
        d=date1
        while d <= date2:
            filename=self.get_varname() + "_" + (str(d).replace("-",""))[0:6]+".nc"
            try:
                glob.glob(inputdir+"/"+filename)
            except:
                print "the file %s does not exist" % (inputdir+filename+".nc")
                print "the directory %s must contain this file:"%inputdir
                print filename
                sys.exit(0)
            datelist.append(d)
            filelist.append(inputdir+"/"+filename)
            d = d + relativedelta(years=+1)
        
        self.set_filelist(filelist)
        self.set_sdates(datelist)


       #### to be simplified!!!!!!!!!!!!!!!!!!!!!!!
    def write_output(self):
        cdo=Cdo()

        #get the needed variables from the object
        filelist=self.get_filelist()
        sdates=self.get_sdates()
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        nmember=self.get_nmember()

        name=set_output_name(varname)
        units=name[0]
        unit_long=name[1]
        long_name=name[2]

        #create output directory
        outputdir=self.get_outputdir()+"/"+varname+"/"        
        if (not os.path.isdir(outputdir)):
            os.mkdir(outputdir)

            
        #loop over all file
        for i in range(len(filelist)):
            f=filelist[i]
            ftemp=(f.split("/")[-1])
            ftemp=ftemp.split(".")[0]+"_temp.nc"


            #open input and output
            ncdfin=NetCDFFile(f, "r")
            ncdfout=NetCDFFile(ftemp, "w")

            variables=ncdfin.variables.keys()
            allDimNames = ncdfin.dimensions.keys() 
  
            ##### time ######
            write_time(ncdfout, ncdfin)
           
            ##### longitude - latitude ######
            write_lonlat(ncdfout, ncdfin)


            #### variable #####
            var=ncdfout.createVariable(varname, 'f', ("time", "latitude", "longitude",))

             #temporary solution! very bad!
            #to be change!!!!!!!!
            #look for a variable named varname or which is the beginning of varname
            #   #add a case for reconstructed variables (prlr or sfcwind) 
            #   #or find a way to change the name of the variable
            if varname in variables:
                varin = ncdfin.variables[varname]
                varaux=varname
            else:
                varaux=(varname.split("_"))[0]
                if varaux in variables:
                    varin = ncdfin.variables[varaux]
                else:
                    varaux3=""
                    for varaux2 in variables:
                        if varaux2.startswith(varaux):
                            varaux3=varaux2
                    if varaux3 == "":
                        print "there is no variable named %s or %s or starting by %s in the file" % (varname, varaux, varaux)
                        print f
                        print variables
                        sys.exit(0)
                    else:
                        varaux=varaux3
                        varin = ncdfin.variables[varaux]

            shapin=shape(varin)
            var.data_type = "float" 
            var.units = units
            var.unit_long = unit_long
            var.standard_name = varname 
            var.long_name = long_name
            if len(shapin)==4:
                var[:]=numpy.resize(varin,(shapin[0],shapin[2],shapin[3]))
            elif len(shapin)==3:
                var[:]=varin
            else:
                print "the dimension are wrong"
                print sys.exit(0)
            ncdfout.close()
            fileout=varname+"_"+(str(sdates[i]).replace("-",""))[0:6]+".nc"
            shutil.copyfile(ftemp, outputdir+fileout)
            
            print "file saved:"
            print outputdir+fileout

    def monthly_nbday(self, percentile, comp):
        """calculate the monthly percentage of days 
        greater the climatological percentile 
        for all file in filelist.
        Note: the climatological percentile 
        is calculated with all files of filelist
        """
        listmondata=self.create_monthly_list()
        listdatastat=[]
        for data in listmondata:
            datastat=data.seasonal_nbday(percentile, comp)
            listdatastat.append(datastat)

        var0=(listdatastat[0]).get_varname()
        var0=quit_leadtime(var0)
        var0=var0.split("-")
        var1=(listdatastat[-1]).get_varname()
        var1=quit_leadtime(var1)
        var1=var1.split("-")

        varnew=var0[0]+"-"+var1[1]
        for datastat in listdatastat:
            datastat.set_varname(varnew)
            datastat.write_output()
        return 0

    def monthly_percentile(self, percentile):
        """calculate the monthly percentile for all file in filelist"""

        cdo=Cdo()
        
        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        
        for i in range(len(filelist)):
            f=filelist[i]
            fileout=(f.split("/")[-1])
            fileout=fileout.split(".")[0]+"_q"+str(percentile)+".nc"
            cdo.monpctl(percentile, input=f+" -monmin "+f+" -monmax "+f, 
                        output=fileout,
                        options="-r -f nc")

            filelist[i] = fileout
            #when ncdf4 instaled change name of variables in files


        exp_stat=self.duplicate()
        exp_stat.set_filelist(filelist)
        exp_stat.set_sdates(sdates)
        exp_stat.set_freq = relativedelta(months=+1)
        
        listmondata=exp_stat.create_monthly_list()
        for datastat in listmondata:
            datastat.set_varname(varname + "_q" + str(percentile))
            datastat.see()
            datastat.write_output()
            
        return 0

def list_season_obs(dataname, varname, sdate1, sdate2, seaslen):
    listobs=[]
    for imon in range(seaslen):
        d1=sdate1+relativedelta(months=+imon)
        d2=sdate2+relativedelta(months=+imon)
        data=DataObs(dataname, varname)
        data.set_inputdirAuto() #look for the input directory
        data.set_outputdirAuto() #look for the output directory
        data.set_files(d1,d2) #look for the files between date1 and date2 
        data.format_files_nc()
        listobs.append(data)

    return listobs

